# encoding: utf-8
"""Core functionality

.. automodule:: atnlp.core.helpers
    :members:
.. automodule:: atnlp.core.logger
    :members:
.. automodule:: atnlp.core.setup
    :members:

"""
__author__ = "Will Davey"
__email__ = "wedavey@gmail.com"
__created__ = "2018-05-29"
__copyright__ = "Copyright 2018 Will Davey"
__license__ = "MIT https://opensource.org/licenses/MIT"

# EOF