"""Model evaluation

.. automodule:: atnlp.eval.html
    :members:
.. automodule:: atnlp.eval.keras
    :members:
.. automodule:: atnlp.eval.metrics
    :members:
.. automodule:: atnlp.eval.plot
    :members:
.. automodule:: atnlp.eval.table
    :members:

"""